// Updates: 2006.01.13


import javax.microedition.lcdui.*;


/**
 * The Menu class represents a menu that displays items in columns.
 * The constructors compute automatically the preferred number of columns
 * (see <code>getPreferredNumberOfColumns()</code>), but this value can be changed using the 
 * <code>setNbCol(int nbCol)</code> method. The <code>fitsOnScreen()</code> method 
 * is used to check whether all the items fit on the screen.
 * 
 * @author Michel Deriaz
 */
public class Menu extends Canvas { 
  final Font PLAIN = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
  final Font BOLD = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD , Font.SIZE_MEDIUM);
  final int LINE_HEIGHT = 15; // the height of a line
  
  String[] items; 
  String[] itemsFull; 
  boolean bold;
  String title;
  MenuListener listener;
  int nbCol, nbLines;
  int currentLine, currentCol;
  int decalY; // space between the top of the screen and the first item


  /**  
   * Constructs a menu with the given items. The number of columns is computed using the 
   * <code>getPreferredNumberOfColumns()</code> method.
   * @param items the items to show in the menu
   * @param itemsFull the full names of the items. These names are displayed on
   * the top of the menu when the corresponding item is selected. This value
   * can be <code>null</code>
   * @param bold determines whether the current selected item must be displayed
   * in bold
   * @param title the title of this menu, or <code>null</code> if there is no
   * title
   * @param listener the listener to fire when an item is selected
   * @see #getPreferredNumberOfColumns()
   */
  public Menu(String[] items, String[] itemsFull, boolean bold, String title, MenuListener listener) {
    this.items = items;
    this.itemsFull = itemsFull;
    this.bold = bold;
    this.title = title;
    this.listener = listener;
    setNbCol(0);
    decalY = 5;
    if (itemsFull != null) decalY += 20;
    if (title != null) decalY += 20;
  }


  /**
   * This constructor is a shortcut for the other constructor using <code>bold = false</code>
   * and <code>title = null</code>.
   * @see #Menu(String[], String[], boolean, String, MenuListener)
   */
  public Menu(String[] items, String[] itemsFull, MenuListener listener) {
    this(items, itemsFull, false, null, listener);
  }

  
  /**
   * Sets the number of columns of the menu. If the parameter equals 0, the 
   * <code>getPreferredNumberOfColumns()</code> method is used to determine
   * the number of columns.
   * @param nbCol the number of columns, or 0 to compute automatically this
   * number
   * @see #getPreferredNumberOfColumns()
   */
  public void setNbCol(int nbCol) {
    if (nbCol < 1) nbCol = getPreferredNumberOfColumns();
    this.nbCol = nbCol;
    nbLines = (int)Math.ceil((double)items.length/(double)nbCol);
    currentLine = (nbLines-1)/2;
    currentCol = (nbCol-1)/2;
    repaint();
  }
  

  /**
   * Computes the maximum number of columns, according to the current device and
   * the values specified in the constructor.
   * @return the maximum number of columns
   */
  public int getMaxNumberOfColumns() {
    int maxWidth = 0;
    for (int i = 0; i < items.length; i++) {
      if (!bold && PLAIN.stringWidth(items[i]) > maxWidth) maxWidth = PLAIN.stringWidth(items[i]);
      if (bold && BOLD.stringWidth(items[i]) > maxWidth) maxWidth = BOLD.stringWidth(items[i]);
    }
    return getWidth()/(maxWidth+2);
  }
  
  
  /**
   * Computes the preferred number of columns, according to the current device
   * and the values specified in the constructor. The preferred number is the 
   * one that minimizes the number of key strokes to access any specific item.
   * This method will therefore return a number so that the displayed items form 
   * a shape as close as possible of a square.
   * @return the preferred number of columns
   */
   public int getPreferredNumberOfColumns() {
    int best;
    if (items.length <= 3) best = 1;
    else if (items.length <= 6) best = 2;
    else if (items.length <= 13) best = 3;
    else if (items.length <= 24) best = 4;
    else best = (int)Math.sqrt(items.length);
    int max = getMaxNumberOfColumns();
    return (best < max ? best : max);
  }
  
 
  /**
   * Returns whether the items fit on the screen.
   * @return <code>true</code> if the items fit on the screen, otherwise <code>false</code>
   */
  public boolean fitsOnScreen() {
    return (nbCol <= getMaxNumberOfColumns() && decalY + nbLines * LINE_HEIGHT < getHeight());
  }


  public void paint(Graphics g) {
    g.setColor(255, 255, 255);
    int w = getWidth();
    int h = getHeight();
    g.fillRect(0, 0, w, h);
    g.setColor(0, 0, 0);
    g.setFont(BOLD);
    if (title != null) g.drawString(title, w/2, 5, Graphics.TOP | Graphics.HCENTER);
    if (itemsFull != null) {
      if (title == null) g.drawString(itemsFull[currentLine * nbCol + currentCol], w/2, 5, Graphics.TOP | Graphics.HCENTER);
      else g.drawString(itemsFull[currentLine * nbCol + currentCol], w/2, 25, Graphics.TOP | Graphics.HCENTER);
    }
    int index = 0;
    for (int i = 0; i < nbLines; i++) {
      for (int j = 0; j < nbCol; j++) {
        if (index < items.length) {
          if (currentLine == i && currentCol == j) {
            g.setFont(bold ? BOLD : PLAIN);
            g.setColor(255, 0, 0);
          }
          else {
            g.setFont(PLAIN);
            g.setColor(0, 0, 0);
          }
          g.drawString(items[index], (int)((0.5+j)*w/nbCol), decalY + i * LINE_HEIGHT, Graphics.TOP | Graphics.HCENTER);
          index++;
        }
      }
    }
  }
  
  
  protected void keyPressed(int keyCode) {
    int gameAction = getGameAction(keyCode);
    if (gameAction == UP) {
      do {
        currentLine = (currentLine - 1 + nbLines) % nbLines;
      }
      while (currentLine * nbCol + currentCol >= items.length);
    }
    if (gameAction == DOWN) {
      do {
        currentLine = (currentLine + 1) % nbLines;
      }
      while (currentLine * nbCol + currentCol >= items.length);  
    }  
    if (gameAction == LEFT) {
      do {
        currentCol = (currentCol - 1 + nbCol) % nbCol;
      }
      while (currentLine * nbCol + currentCol >= items.length);  
    }  
    if (gameAction == RIGHT) {
      do {
        currentCol = (currentCol + 1) % nbCol;
      }
      while (currentLine * nbCol + currentCol >= items.length);  
    }
    if (gameAction == FIRE) listener.menuItemChoosen(currentLine * nbCol + currentCol);
    repaint();
  }
}